//////////////////////////////////////////////////////////////////
//
// WaitingARQ.cxx
//
// This work is published under the GNU Public License (GPL)
// see file COPYING for details.
// We also explicitely grant the right to link this code
// with the OpenH323 library.
//
// initial author: Dennis Lazreg
//
//////////////////////////////////////////////////////////////////

#if HAS_WAITARQ

#if (_MSC_VER >= 1200)  
#pragma warning( disable : 4800 ) // one performance warning off
#pragma warning( disable : 4786 ) // warning about too long debug symbol off
#define snprintf _snprintf
#endif

#include "WaitingARQ.h"
#include "RasSrv.h"
#include "h323pdu.h"



bool WaitingARQlist::RouteToAlias( PString agent, PString epId, PString callRef )
{
	iterator Iter = FindByEndpointIdAndCallRef ( epId, atoi(callRef) );
	if (Iter == wArqList.end()) {
		PTRACE(2, "RouteToAlias: call not found");
		// TODO: signal status port
		return false;
	} else {
		H225_RasMessage &obj_rr = (*Iter)->m_ARQ;
		H225_AdmissionRequest &obj_arq	= obj_rr;

		PIPSocket::Address rx_addr = (*Iter)->m_rx_addr ;
		H323SetAliasAddress(agent, obj_arq.m_destinationInfo[0]);		 

		RasThread->DoARQ (rx_addr, obj_rr);

		delete *Iter ;
		wArqList.erase(Iter);
		return true;
	}
}


bool WaitingARQlist::RouteReject(PString SourceEpId, PString CallRef )
{
	iterator Iter = FindByEndpointIdAndCallRef (SourceEpId, atoi(CallRef) );
	if (Iter == wArqList.end()) {
		PTRACE(2, "RouteReject: call not found");
		// TODO: signal status port
		return false;
	} else {
		(*Iter)->DoARJ();
		delete *Iter;
		wArqList.erase(Iter);
		return true;
	}
}

 
bool WaitingARQlist::InsertWaitingARQ (const H225_RasMessage & obj_arq, const PIPSocket::Address & rx_addr  , const PString &ccQueue  , const endptr & requestingEP)
{
	const H225_AdmissionRequest & obj_rr = obj_arq;

	int callRef =   obj_rr.m_callReferenceValue;

	iterator Iter = FindByCallRefNumber ( callRef );
	if (Iter == wArqList.end()) {
	  PTRACE(2, "InsertWaitingARQ in WaitingARQlist: " << (unsigned)callRef );
		PWaitAndSignal lock(usedLock);
 		wArqList.push_back (new WaitingARQ (sequenceNumber, obj_arq, rx_addr , ccQueue , requestingEP ));
		++sequenceNumber;
		return true;
	} else {
		 PString  epId   = (*Iter)->getEndPointIdentifier () ;
		 PString  cq		 = (*Iter)->getAgentQueue () ; 
		 PTRACE(2, "InsertWaitingARQ in WaitingARQlist: Already inserted with callRef: " << callRef << " epId=" << epId << " Virtual Queue:" << cq );
		 return false ;
	}
}
 
 
void WaitingARQlist::CheckWaitingARQ()
{
	PWaitAndSignal lock(usedLock);
	iterator Iter = find_if(wArqList.begin(), wArqList.end(), not1(bind2nd(mem_fun(&WaitingARQ::IsTimeOut), waitingTTL)));
	for_each(wArqList.begin(), Iter, mem_fun(&WaitingARQ::DoARJ) );
	for_each(wArqList.begin(), Iter, deleteobj<WaitingARQ>() );
	wArqList.erase(wArqList.begin(), Iter);
}

#endif // HAS_WAITARQ
